package com.jpattern.gwt.client.navigationevent;

import java.util.HashMap;
import java.util.Map;

import com.jpattern.gwt.client.IApplicationProvider;
import com.jpattern.gwt.client.history.IHistoryManager;
import com.jpattern.gwt.client.logger.ILogger;
import com.jpattern.gwt.client.presenter.APresenter;
import com.jpattern.gwt.client.presenter.IPresenter;
import com.jpattern.gwt.client.presenter.NullHierarchyStrategy;
import com.jpattern.gwt.client.presenter.NullPresenter;
import com.jpattern.gwt.client.view.IView;
import com.jpattern.gwt.client.view.NullHasWidgets;
import com.jpattern.gwt.client.view.NullShowViewStrategy;
import com.jpattern.gwt.client.view.NullView;

/**
 * 
 * @author Francesco Cina
 *
 * 03/ago/2011
 */
public class RootNavigationEvent extends ANavigationEvent {

	private final INavigationEvent mainNavigationEvent;

	public RootNavigationEvent(INavigationEvent mainNavigationEvent ) {
		super("");
		this.mainNavigationEvent = mainNavigationEvent;
	}
	
	public RootNavigationEvent(INavigationEvent mainNavigationEvent, IApplicationProvider applicationProvider) {
		super("" , applicationProvider);
		this.mainNavigationEvent = mainNavigationEvent;
	}
	
	/**
	 * Start the client GWT application
	 */
	public IPresenter startApplication() {
		return launch(new NullPresenter(), new HashMap<String, String>(), new String[0]);
	}

	@Override
	protected IPresenter exec(Map<String, String> queryStringValues) {
		ILogger logger = getProvider().getLoggerService().getLogger(getClass());
		logger.debug("exec", "root presenter generated");
		IPresenter rootPresenter = new RootPresenter(this);
		rootPresenter.setApplicationProvider(getProvider());
		IHistoryManager historyManager = getProvider().getHistoryService().getHistoryManager();
		historyManager.setRootPresenter(rootPresenter);
		historyManager.updateState();
		return rootPresenter;
	}
	
	class RootPresenter extends APresenter<IView> {
		
		ILogger logger;

		public RootPresenter(INavigationEvent navigationEvent) {
			super(new NullShowViewStrategy<IView>(new NullView()), navigationEvent);
			setHierarchyStrategy(new NullHierarchyStrategy());
			registerNavigationEvent(mainNavigationEvent, new NullHasWidgets(), new NullNavigationEventCallback());
			ready();
		}

		@Override
		public void preRender() {
		}

		@Override
		public void init() {
		}

		@Override
		public void onUnloadPresenter() {
		}
	}
	
}
